package com.example.android.clouddb;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Observable;
import java.util.Observer;


public class DBDisplay extends Fragment implements Observer {

    private ListView displayListView;
    private ArrayAdapter<String> listAdapter;
    private DBHandler dbHandler;
    private static final String DB_HANDLER_KEY = "handler";
    private HashMap<String, Recipe> updatedData = new HashMap<>();
    private ArrayList<String> arrayList;
    private Context appContext;


    public DBDisplay() {}

    public static DBDisplay newInstance(DBHandler handler) {
        Bundle bundle = new Bundle();
        bundle.putParcelable(DB_HANDLER_KEY, handler);
        DBDisplay fragment = new DBDisplay();
        handler.addObserver(fragment);
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        dbHandler = (DBHandler) getArguments().getParcelable(DB_HANDLER_KEY);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_dbdisplay_list, container, false);
        displayListView = (ListView) view.findViewById(R.id.db_display_list_view);
        return view;
    }

    @Override
    public void onStart() {
        super.onStart();
        showData(updatedData, appContext);
    }
    @Override
    public void update(Observable observable, Object o) {
        populateDisplayListOnContext(appContext);
    }

    public void setInitialData(HashMap initialData) {
        updatedData = initialData;
    }

    public void populateDisplayListOnContext(Context aContext) {
        arrayList = new ArrayList<String>(updatedData.keySet());
        listAdapter = new ArrayAdapter<String>(aContext, R.layout.db_item_breif, arrayList);
        displayListView.setAdapter(listAdapter);

        displayListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int position, long id) {
                inflateEditView(adapterView.getItemAtPosition(position).toString());
            }
        });
    }

    public void showData(HashMap data, Context context) {
        updatedData = data;
        populateDisplayListOnContext(context);
    }

    public void clearDisplay() {
        showData(new HashMap<String, Recipe>(), appContext);
    }
    public ArrayList<String> getDisplayedRecipies() {
        return new ArrayList<String>(updatedData.keySet());
    }

    public void showRecipe(Recipe recipe) {
        HashMap<String, Recipe> valid = new HashMap<>();
        valid.put(recipe.getRecipeName(), recipe);
        showData(valid, appContext);
    }

    public void filterByIngredient(String ingredient) {
        updatedData = dbHandler.getRecipes();
        HashMap<String, Recipe> valid = new HashMap<>();
        for (String key : updatedData.keySet()) {
            Recipe recipe = updatedData.get(key);
            if (recipe.hasIngredient(ingredient)) {
                valid.put(recipe.getRecipeName(), recipe);
            }
        }
        showData(valid, appContext);
    }
    public void setAppContext(Context context) {
        appContext = context;
    }

    public void getRecipesForKeyword(String keyword) {
        updatedData = dbHandler.getRecipes();
        HashMap<String, Recipe> valid = new HashMap<>();
        for (String key : updatedData.keySet()) {
            Recipe recipe = updatedData.get(key);
            if (recipe.containsKeyword(keyword)) {
                valid.put(recipe.getRecipeName(), recipe);
            }
        }
        System.out.println("RPW - found: " + valid.keySet().toString());
        showData(valid, appContext);
    }

    public void getRecipesForMeal(int mealKey) {

        updatedData = dbHandler.getRecipes();
        HashMap<String, Recipe> valid = new HashMap<>();
        for (String key : updatedData.keySet()) {
            Recipe recipe = updatedData.get(key);
          }
        showData(valid, appContext);
    }

    public void inflateEditView(String recipeName){
        FragmentManager manager = getFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        Fragment editFragment = EditRecipe.newInstance(recipeName, dbHandler);
        transaction
                .replace(R.id.fragment_container, editFragment)
                .addToBackStack(null)
                .commit();

    }
    public DBHandler getDatabase() {
        return dbHandler;
    }
    public void setDatabase(DBHandler database) {
        dbHandler = database;
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        appContext = context;
    }

    @Override
    public void onDetach() {
        super.onDetach();
    }

}
