package com.example.android.clouddb;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ListView;

import java.util.HashMap;


public class EditRecipe extends Fragment {
    private static final String RECIPE_NAME_KEY = "recipeName";
    private static final String DB_HANDLER_KEY = "dbHandler";
    private String recipeName;
    private DBHandler dbHandler;

    // Layout components
    private Button saveAndClose;
    private EditText recipeNameEntry;
    private EditText recipeDescriptionEntry;
    private ListView ingredientDisplay;
    private EditText ingredientAmount;
    private EditText ingredientName;
    private CheckBox boolVegetarian;
    private Button addIngredient;

    // Ingredients dynamic display adapters
    private MapAdapter<String, Integer> ingredientsAdapter;
    private HashMap<String, Integer> ingredients;

    public EditRecipe() {}

    public static EditRecipe newInstance(String recipeID, DBHandler dbHandler) {
        EditRecipe fragment = new EditRecipe();
        Bundle args = new Bundle();
        args.putString(RECIPE_NAME_KEY, recipeID);
        args.putParcelable(DB_HANDLER_KEY, dbHandler);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            recipeName = getArguments().getString(RECIPE_NAME_KEY);
            dbHandler = getArguments().getParcelable(DB_HANDLER_KEY);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_edit_recipe, container, false);

        indentifyLayoutComponents(view);  // Initializes layout component variables by IDs

        Recipe recipe = dbHandler.getRecipeByID(recipeName);  // Retrieve requested recipe
        fillRecipeData(recipe);  // Populate screen with data from recipe


        saveAndClose.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Recipe oldRecipe = dbHandler.getRecipeByID(recipeName);
                Recipe recipe = getRecipeFromForm();
                dbHandler.replaceRecipe(oldRecipe, recipe);
                getFragmentManager().popBackStackImmediate();
            }
        });

        return view;
    }

    public Recipe getRecipeFromForm() {
        String name = recipeNameEntry.getText().toString();
        String description = recipeDescriptionEntry.getText().toString();
        Boolean isVegetarian = boolVegetarian.isChecked();
        return new Recipe(name, description, ingredients, isVegetarian);
    }

    // Look up layout components and save references to global vars
    public void indentifyLayoutComponents(View view) {
        ingredientDisplay = (ListView) view.findViewById(R.id.show_ingredient_list);
        recipeNameEntry = (EditText) view.findViewById(R.id.item_name);
        recipeDescriptionEntry = (EditText) view.findViewById(R.id.item_description);
        saveAndClose = (Button) view.findViewById(R.id.action_save_recipe);
        ingredientDisplay = (ListView) view.findViewById(R.id.show_ingredient_list);
        addIngredient = (Button) view.findViewById(R.id.add_ingredient_trigger);
        ingredientAmount = (EditText) view.findViewById(R.id.ingredient_amount_entry);
        ingredientName = (EditText) view.findViewById(R.id.ingredient_name_entry);
        boolVegetarian = (CheckBox) view.findViewById(R.id.is_vegetarian);

    }

    // update display with data from recipe
    public void fillRecipeData(Recipe recipe) {
        recipeNameEntry.setText(recipe.getRecipeName());
        recipeDescriptionEntry.setText(recipe.getInstructions());
        ingredients = recipe.getIngredients();
        boolVegetarian.setChecked(recipe.hasTag("vegetarian"));
        fillIngredientList();


        // Handle plus ingredient button click
        addIngredient.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View view) {
                String key = ingredientName.getText().toString();
                int value = Integer.valueOf(ingredientAmount.getText().toString());
                addToIngredientList(key, value); // also refreshes ingredient display adapter
                clearIngredientForm();
            }
        });
    }

    public void clearIngredientForm() {
        ingredientAmount.setText("");
        ingredientName.setText("");
    }

    public void addToIngredientList(String key, int value) {
        ingredientsAdapter = null;
        ingredients.put(key, value);
        ingredientsAdapter = new MapAdapter<String, Integer>(this.getContext(), ingredients);
        ingredientDisplay.setAdapter(ingredientsAdapter);
    }

    // Populate Ingredient List
    public void fillIngredientList() {
        ingredientsAdapter = new MapAdapter<String, Integer>(this.getContext(), ingredients);
        ingredientDisplay.setAdapter(ingredientsAdapter);
    }

}
