package com.example.android.clouddb;

import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListAdapter;
import android.widget.ListView;

import java.util.ArrayList;
import java.util.HashMap;


public class GroceryDisplay extends Fragment {

    private ListView displayListView;
    private ListAdapter listAdapter;
    private ArrayList<String> groceryList;
    private HashMap<String, Integer> groceryHash;
    private DBHandler database;


    public GroceryDisplay() { }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.database = ((MainActivity) getActivity()).getDatabase();
        this.groceryHash = new HashMap<String, Integer>();
        this.groceryList = generateList();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_grocery_display, container, false);
        displayListView = (ListView) view.findViewById(R.id.grocery_display_list_view);
        populateDisplayListOnContext(getContext());
        return view;

    }


    public void populateDisplayListOnContext(Context aContext) {
        listAdapter = new ArrayAdapter<String>(aContext, R.layout.db_item_breif, groceryList);
        displayListView.setAdapter(listAdapter);
    }

    public ArrayList<String> mapToList (HashMap<String, Integer> aMap) {
        ArrayList<String> list = new ArrayList<String>();
        for (String key : aMap.keySet()) {
            list.add(aMap.get(key) + "g    " + key);
        }
        return list;
    }

    public ArrayList<String> getGroceryList() {
        return this.groceryList;
    }

    public void addRecipeToList(Recipe recipe) {
        HashMap<String, Integer> ingredients = recipe.getIngredients();
        for (String ingredient : ingredients.keySet()) {
            int amount = ingredients.get(ingredient);
            if (groceryHash.containsKey(ingredient)) {
                amount += groceryHash.get(ingredient);
            }
            groceryHash.put(ingredient, amount);
        }
    }


    public void addMealToList(int mealID) {
        HashMap<String, Recipe> mealRecipes = database.getRecipesWithMealTag(mealID);
        for (String recipeName : mealRecipes.keySet()) {
            addRecipeToList(mealRecipes.get(recipeName));
        }
    }

    public ArrayList<String> generateList() {
        for (int mealID = 0; mealID < 3; mealID++) {
            addMealToList(mealID);
        }
        return mapToList(this.groceryHash);
    }


}
