package com.example.android.clouddb;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;

import java.util.ArrayList;
import java.util.HashMap;


public class MealPlanner3 extends Fragment {

    private DBHandler dbHandler;
    private MainActivity appActivity;
    private DBDisplay breakfast;
    private DBDisplay lunch;
    private DBDisplay dinner;
    private ArrayList<HashMap<String, Recipe>> mealPlan;

    public MealPlanner3() { }

    @Override
    public void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        mealPlan = new ArrayList<HashMap<String, Recipe>>();
        for (int i = 0; i < 3; i++) {
            mealPlan.add(i, new HashMap<String, Recipe>());
        }
   }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_meal_planner, container, false);

        appActivity = (MainActivity) getActivity();
        dbHandler = appActivity.getDatabase();

        // Inflate view for each meal
        breakfast = initPlan(R.id.breakfast_results_frame, R.id.action_generate_breakfast, view, 0);
        lunch = initPlan(R.id.lunch_results_frame, R.id.action_generate_lunch, view, 1);
        dinner = initPlan(R.id.dinner_results_frame, R.id.action_generate_dinner, view, 2);

        enableSaveMealAction(view);
        return view;
    }


    public void enableSaveMealAction(View view){
        Button button = (Button) view.findViewById(R.id.action_save_plan);
        button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                updateMealPlan(0, breakfast);
                updateMealPlan(1, lunch);
                updateMealPlan(2, dinner);
                closeMealPlanner();


            }
        });
    }

    public void closeMealPlanner() {
        // Remove meal displays from stack
        System.out.println("RPW - pop stack: " + getFragmentManager().getBackStackEntryCount());
        for (int i = 0; i < 3; i++){
            getFragmentManager().popBackStackImmediate();
        }
        // Remove self
        getFragmentManager().popBackStack();
    }

    public void updateMealPlan(int mealID, DBDisplay display) {
        ArrayList<String> oldMeal = new ArrayList(dbHandler.getRecipesWithMealTag(mealID).keySet());
        dbHandler.updateMealPlan(mealID, oldMeal, false);
        dbHandler.updateMealPlan(mealID, display.getDisplayedRecipies(), true);
    }

    public DBDisplay initPlan(int planID, int buttonID, View view, int mealID){
        Button button = (Button) view.findViewById(buttonID);
        DBDisplay display = DBDisplay.newInstance(dbHandler);
        appActivity.replaceFragment(display, planID);
        setAddMealOnClick(button, display, mealID);
        displayMeal(display,mealID);
        return display;
    }


    public void displayMeal(DBDisplay display, int mealID) {
        HashMap<String, Recipe> subset = dbHandler.getRecipesWithMealTag(mealID);
        mealPlan.add(mealID, subset);
        display.setInitialData(subset);
    }


    public void addMealToPlan(DBDisplay plan, final int planID) {
        Recipe recipe = dbHandler.getRandomRecipe();
        if (recipe == null) {
            return;
        }
        if (!mealPlan.contains(planID)) {
            mealPlan.add(planID, new HashMap<String, Recipe>());
        }
        mealPlan.get(planID).put(recipe.getRecipeName(), recipe);
        plan.showData(mealPlan.get(planID), this.getContext());
    }


    private void setAddMealOnClick(final Button btn, final DBDisplay display, final int planID){
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                addMealToPlan(display, planID);
            }
        });
    }


}
