package com.example.android.clouddb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Stream;

public class Recipe {

    private String recipeName;
    private String instructions;
    private HashMap<String, Integer> ingredients;
    private ArrayList<Boolean> mealTags;
    private ArrayList<String> tags;

    public Recipe () {
        init("", "");
    }

    public Recipe(String recipeName, String instructions) {
        init(recipeName, instructions);
    }

    public Recipe(String recipeName, String instructions, HashMap<String, Integer> ingredientMap) {
        init(recipeName, instructions);
        this.ingredients = ingredientMap;
     }

    public Recipe(String recipeName, String instructions, HashMap<String, Integer> ingredientMap, boolean isVegetarian) {
        init(recipeName, instructions);
        this.ingredients = ingredientMap;
        if (isVegetarian) {
            tags.add("vegetarian");
        }
    }

    private void init(String name, String instructions) {
        this.recipeName = name;
        this.instructions = instructions;
        this.ingredients = new HashMap<String, Integer>();
        this.mealTags = new ArrayList<Boolean>();
        this.tags = new ArrayList<String>();
        for (int i = 0; i < 3; i++) {
            mealTags.add(false);
        }
    }


    public String getRecipeName() {
        return recipeName;
    }

    public String getInstructions() {
        return instructions;
    }

    public HashMap<String, Integer> getIngredients() {
        return ingredients;
    }

    public ArrayList<Boolean> getMealTags() { return this.mealTags; }

    public void setRecipeName(String recipeName) {
        this.recipeName = recipeName;
    }

    public void setInstructions(String instructions) {
        this.instructions = instructions;
    }

    public void setIngredients(HashMap<String, Integer> ingredients) {
        this.ingredients = ingredients;
    }

    public void setTags(ArrayList<String> tags) {
        this.tags = tags;
    }

    public ArrayList<String> getTags() {
        return this.tags;
    }

    public void setMealTags (ArrayList<Boolean> mealTags) { this.mealTags = mealTags; }

    public void addIngredient(String name, int amount) {
        String key = name.toLowerCase();
        if (ingredients.containsKey(key)) {
            int oldAmount = ingredients.get(key);
            ingredients.put(key, amount + oldAmount);
        }
    }

    public void addTag(String tag) {
        this.tags.add(tag);
    }
    public boolean hasTag(String tag) {
        return this.tags.contains(tag);
    }

    public boolean satisfiesTags(Set<String> requiredTags) {
        for (String tag : requiredTags) {
            if (! hasTag(tag)) {
                return false;
            }
        }
        return true;
    }

    private String getFlattenedRecipe(){

        StringBuilder builder = new StringBuilder();
        builder.append(recipeName).append(" ");
        builder.append(instructions).append(" ");
        builder.append(ingredients.toString()).append(" ");
        builder.append(tags.toString());
        String flattenedRecipe = builder.toString().toLowerCase();
        return flattenedRecipe;
    }

    public boolean isTaggedForMeal(int mealKey) {
        return mealTags.get(mealKey);
    }

    public boolean containsKeyword(String keyword) {
        return getFlattenedRecipe().contains(keyword.toLowerCase());
    }

    public void tagMeal(int mealID, Boolean tagValue) {
        mealTags.set(mealID, tagValue);
    }


    public boolean hasIngredient(String ingredient) {
        return ingredients.containsKey(ingredient.toLowerCase());
    }
    
}